@@ -4,4 +4,4 @@
 # Avoid R CMD check : 'no visible binding for global variable' NOTE
 #' @importFrom utils globalVariables
 #' @keywords internal
-utils::globalVariables(c("..density..", "..prop..", "Freq", "Var1", "Var3", "m.set", "obs", "NA.condition"))
+utils::globalVariables(c("density", "prop", "Freq", "Var1", "Var3", "m.set", "obs", "NA.pattern"))
