@@ -1,8 +1,13 @@
 # Checking for pmm.type and bootstrap constraints
-check_pmm <- function(pmm.type, bootstrap, xgb.params, Nrow, sorted.naSums, sorted.types, pmm.k) {
-  # without bootstrapping: at least one of the sample related hyperparameters in xgboost must be less than 1
+check_pmm <- function(pmm.type, bootstrap=FALSE, xgb.params, Nrow, sorted.naSums, sorted.types, pmm.k) {
+  # without bootstrapping: at least one of the sampling-related hyperparameters in xgboost must be less than 1
   if (!is.null(pmm.type)) {
-    ################
+    if (!pmm.type %in% c(0, 1, 2, "auto")) {
+      stop("The specified pmm.type is incorrect. It must be one of the following types: NULL,1,2,\"auto\".")
+    }
+
+
+    ## pmm.type=1 or pmm.type=2 and without bootstrap:  requires subsample<1
     if ((pmm.type == 1 | pmm.type == 2) & bootstrap == FALSE) {
       sample.params <- grepl("sample", names(xgb.params))
       if (all(sample.params == FALSE)) {
@@ -15,39 +20,23 @@ check_pmm <- function(pmm.type, bootstrap, xgb.params, Nrow, sorted.naSums, sort
       }
     }
 
-    #################
+    ## pmm.type=0 does not allow bootstrap
     if (pmm.type == 0 & bootstrap == TRUE) {
-      stop("PMM type0 requires using the whole dataset. Bootstrap or sample hyperparameters<1 are not allowed.")
+      stop("PMM type0 requires using the whole dataset. Bootstrap or sampling-related hyperparameters<1 are not allowed. All the following hyperparameters must be 1:\n subsample\n colsample_bytree\n colsample_bylevel\n colsample_bynode.")
     }
 
+
+    ## pmm.type=0 does not allow subsampling
     if (pmm.type == 0 & bootstrap == FALSE) {
       sample.params <- grepl("sample", names(xgb.params))
       if (any(sample.params == TRUE)) {
         if (any(xgb.params[sample.params] != 1)) {
-          stop("PMM type0 requires using the whole dataset. Bootstrap or sample hyperparameters<1 are not allowed.")
-        }
-      }
-    }
-    ###########
-    if (!pmm.type %in% c(0, 1, 2, "auto")) {
-      stop("The specified pmm.type is incorrect. It must be one of the following types: NULL,1,2,\"auto\".")
-    }
-    ########
-    if (pmm.type == 0) {
-      if (bootstrap == TRUE) {
-        stop("PMM type0 need to use the whole dataset. It does not allow bootstrap.")
-      } else {
-        # bootstrap=FALSE, but need to check other sample related params to be 1
-        sample.params <- grepl("sample", names(xgb.params))
-        if (any(sample.params == TRUE)) {
-          if (any(xgb.params[sample.params] != 1)) {
-            stop("For multiple imputation through XGBoost with PMM type 0 (without bootstrap), all the following hyperparameters must be 1:\n subsample\n colsample_bytree\n colsample_bylevel\n colsample_bynode")
-          }
+          stop("PMM type0 requires using the whole dataset. Bootstrap or sampling-related hyperparameters<1 are not allowed. All the following hyperparameters must be 1:\n subsample\n colsample_bytree\n colsample_bylevel\n colsample_bynode.")
         }
       }
     }
 
-    # .......................
+    ## if pmm.type=0,1 or 2, all variables need to perform PMM
     if (any(Nrow - sorted.naSums < pmm.k) && pmm.type != "auto") {
       maxNA <- max(sorted.naSums)
       minObs <- Nrow - maxNA
@@ -60,7 +49,7 @@ check_pmm <- function(pmm.type, bootstrap, xgb.params, Nrow, sorted.naSums, sort
       }
       stop(paste(s1, s2, s3, sep = "\n"))
     }
-    ### if pmm.type="auto", only numeric variables need to perform PMM
+    ## if pmm.type="auto", only numeric variables need to perform PMM
     if (pmm.type == "auto") {
       idx <- which(Nrow - sorted.naSums < pmm.k & sorted.types == "numeric")
       if (length(idx) > 0) {
